# encoding: utf-8
'''
Created on 2016-6-23

@author: xiewei

根据自定义包集合发布软件源

'''

from ..contrib import docopt
import os
import subprocess
import logging
from .config import options

logger = logging.getLogger('archive_man')

cmd_doc = """
Usage:
   archive_man publish <topdir> [-s <suite>] [-v <version>] [-a <architecture>...] [-d <description>]

Options:
   -h, --help              show this help.
   -s,--suite=<suite>      set codename of archive. [default: %(suite)s]
   -v,--version=<version>  set version info of archive [default: 1.0].
   -a,--architecture=<architecture>
                           set architectures in archive, May be specified
                           multiple times. Specially, add 'src' in architectures
                           can generate sources index.
                           [default: %(arch)s]
   -d,--description=<description>
                           set description in Release.
""" % options


def _verify_args(args):
    data = {}
    data['Architectures'] = ' '.join(args['--architecture'])
    data['Version'] = args['--version']
    data['Suite'] = args['--suite']
    data['Codename'] = args['--suite']
    data['Components'] = 'main'
    data['Description'] = args.get('--description', 'Customized archive.')
    data['topdir'] = os.path.abspath(os.path.expanduser(args['<topdir>']))
    return data


def gen_packages(topdir, dist, arch, component='main'):
    logger.info('generating Packages file for %s', arch)
    if arch == 'src':
        index_dir = os.path.join(topdir, 'dists', dist,
                                component, 'source')
        if not os.path.exists(index_dir):
            os.makedirs(index_dir)
        packagefile = os.path.join(index_dir, 'Sources')
        cmd = 'apt-ftparchive sources pool > "%s"' % packagefile
    else:
        index_dir = os.path.join(topdir, 'dists', dist,
                                component, 'binary-' + arch)
        if not os.path.exists(index_dir):
            os.makedirs(index_dir)
        packagefile = os.path.join(index_dir, 'Packages')
        cmd = 'apt-ftparchive --arch=%s packages pool > "%s"' % (arch, packagefile)
    ret = subprocess.call(cmd,
                          cwd=topdir, shell=True
                          )
    if ret == 0:
        from .utils import strip_packages
        strip_packages(packagefile)  # will also compress Packages file
    return ret == 0


def gen_release(topdir, data):
    logger.info('generating Release file')
    data['Origin'] = 'apt-archive'
    data['Label'] = 'Apt Archive'

    conf = """APT::FTPArchive::Release {
Origin "%(Origin)s";
Label "%(Label)s";
Suite "%(Suite)s";
Codename "%(Codename)s";
Version "%(Version)s";
Architectures "%(Architectures)s";
Components "%(Components)s";
Description "%(Description)s";
};
""" % data
    # write conf
    import tempfile
    tmpconf = tempfile.mktemp('.conf')
    with open(tmpconf, 'w') as f:
        f.write(conf)
    # generate Release
    os.system(
        'rm -f "%(top)s"/InRelease "%(top)s"/Release.gpg "%(top)s"/Release' % {'top': topdir})
    content = os.popen('apt-ftparchive -c %(conf)s release %(top)s' % {'conf': tmpconf,
                                                                       'top': topdir
                                                                       }
                       ).read()
    with open(os.path.join(topdir, 'Release'), 'w') as f:
        f.write(content)
    logger.info('built Release in %s' % topdir)
    from .sign import sign_file
    sign_file(topdir)


def publish_archive(data):
    pool = os.path.join(data['topdir'], 'pool')
    dists = os.path.join(data['topdir'], 'dists', data['Suite'])
    if not os.path.isdir(pool):
        logger.error('%s is not a directory' % pool)
        return 1
    if not os.path.exists(dists):
        os.makedirs(dists)
    # generate packages
    component = data['Components'].split()[0]
    for arch in data['Architectures'].split():
        if gen_packages(topdir=data['topdir'],
                        dist=data['Suite'],
                        arch=arch,
                        component=component
                        ):
            continue
        else:
            return 1

    # generate release
    gen_release(dists, data)
    logger.info('archive published. Source: deb file://%s %s %s' % (data['topdir'],
                                                                    data['Suite'],
                                                                    component
                                                                    )
                )
    return 0


def main(argv=None):
    """
    publish a customized archive form a package pool
    """
    args = docopt.docopt(cmd_doc, argv, help=True)
    ret = publish_archive(_verify_args(args))
    return ret



import logging

logger = logging.getLogger('archive_man')
hdlr = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr)
logger.setLevel(logging.DEBUG)



# encoding: utf-8
'''
Created on 2019-01-22

@author: xiewei
'''

import os
import sys
from ..contrib import gnupg
from ..contrib import docopt
from . import config
from . import logging

logger = logging.getLogger('archive_man')

cmd_doc = """
Usage:
   archive_man sign <file> [-p <password>]

Options:
   -h, --help  show this help
   -p, --password=<password>    the password of the imported signing key,
                                if you need and didn't set it in the config file

This command help user sign the "Release" file of a apt archive,
the signed file will be "InRelease" and "Release.gpg".

if <file> is a directory, then sign "<dir>/Release".
if <file> is a normal file, then directly sign it, even the file name is not "Release".
"""


def sign_file(filepath, password=None, restrict=False):
    gpg = gnupg.GPG(gnupghome=config.GPGHOME)
    if not gpg.list_keys(secret=True):
        error_str = '尝试签名出错，未导入私钥，请先执行 %s key import' % sys.argv[0]
        if restrict:
            logger.error(error_str)
            sys.exit(1)
        else:
            logger.warning(error_str)
            return 1
    if os.path.isdir(filepath):
        release_file = os.path.join(filepath, 'Release')
        fn = 'Release'
    else:
        release_file = filepath
        filepath, fn = os.path.split(release_file)

    if not os.path.exists(release_file):
        error_str = 'File not exist: %s'
        if restrict:
            logger.error(error_str, release_file)
            sys.exit(1)
        else:
            logger.warning(error_str, release_file)
            return 1

    data = open(release_file, 'rb').read()
    if password is None:
        password = config.GPGKEYPASS
    result = gpg.sign(data, passphrase=password,
                      clearsign=True, binary=False,
                      algo='sha512'
                      )
    assert result.data, result.stderr
    with open(os.path.join(filepath, 'In' + fn), 'wb') as f:
        f.write(result.data)

    result = gpg.sign(data, passphrase=password,
                      clearsign=False, binary=True,
                      algo='sha512'
                      )
    assert result.data, result.stderr
    with open(os.path.join(filepath, fn + '.gpg'), 'wb') as f:
        f.write(result.data)
    logger.info('Release signed')
    return 0


def main(argv=None):
    """
    sign Release file with imported PGP key
    """
    args = docopt.docopt(cmd_doc, argv, help=True)
    return sign_file(args['<file>'], password=args['--password'],restrict=True)



#!/usr/bin/python
# encoding: utf-8
'''
archive_man -- apt archive manage tool

archive_man is a manage tool of APT software archives

@author:     Xie Wei
        
@copyright:  2016 KylinOS. All rights reserved.
        
@license:    GPL

@contact:    xiewei@kylinos.cn
@deffield    updated: Updated
'''

from __future__ import print_function
import os
import sys
from .contrib import docopt

__all__ = []

try:
    __version__ = open(os.path.join(os.path.dirname(__file__),
                                    '../VERSION')
                       ).read().strip()
    DEVELOP = True
except Exception:
    import pkg_resources
    __version__ = pkg_resources.get_distribution('apt-archive-tools').version
    DEVELOP = False
__date__ = '2016-06-14'
__updated__ = '2019-02-15'


def main():
    """cmd line entry"""

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__

    cmd_doc = """
Usage: %(prog)s [--help] [--quiet] <command> [<args>...]

options:
   -h, --help   Show this screen and exit.
   -q, --quiet  Do not show debug message.
Command list:
"""
    commands = {}
    if DEVELOP:
        import pkgutil
        import importlib
        from . import lib

        for _finder, name, _ispkg in pkgutil.iter_modules(lib.__path__, lib.__name__ + '.'):
            module = importlib.import_module(name)
            if hasattr(module, 'main'):
                if hasattr(module, 'command'):
                    command = module.command
                else:
                    command = name.split('.')[-1]
                commands[command] = module.main
    else:
        for entry_point in pkg_resources.iter_entry_points('apt_archive_tools.commands'):
            commands[entry_point.name] = entry_point.load()

    for c, f in commands.items():
        cmd_doc = cmd_doc + '   ' + c + '\n\t' + \
            (f.__doc__ or 'No description').strip() + '\n'

    cmd_doc += "\nSee '%(prog)s <command> --help' for more information on a specific command.\n"

    args = docopt.docopt(cmd_doc % {'prog': program_name},
                         help=True,
                         version=program_version,
                         options_first=True
                         )
    if args['--quiet']:
        from .lib import logger, logging
        logger.setLevel(logging.INFO)

    command = args['<command>']

    if command not in commands:
        print("%r is not a valid command. See '%s --help'." %
              (command, program_name))
        sys.exit(1)
    sys.exit(commands[command]([command] + args['<args>']))



# coding:utf-8

'''
Created on 2017-03-24

@author: xiewei
'''

cmd_doc = """
检查软件源中是否存在依赖未满足的包
Usage: archive_man checkdep <suite> [-e <dependency_suite>...] [--ignore-noexist] [--ge-only]

suite: 软件源索引目录，里面应该有Release文件

options:
   --ignore-noexist   忽略未找到的依赖包,只显示版本号不满足的
   --ge-only          只对比要求>=的依赖，因为编译环境错误而导致的依赖偏差应该都是这种形式
   -e,--extra=<dependency_suite>  添加额外的源用于查找依赖

"""

import os
from ..contrib import docopt
from . import utils

import logging

logger = logging.getLogger('archive_man')


class DependMatch(Exception):
    pass


def checkdep(topdir, extra, ignore_noexist=False, ge_only=False):
    """
    查找依赖未满足的包
    """
    pkg_in_archive = {}
    release = utils.Release.parse(os.path.join(topdir, 'Release'))
    extra_releases = [utils.Release.parse(
        os.path.join(extradir, 'Release')) for extradir in extra]
    # find all Packages

    for r in [release] + extra_releases:
        for packages in r.all_packages.values():
            for pkg in packages:
                # package
                if pkg.name not in pkg_in_archive:
                    pkg_in_archive[pkg.name] = [pkg]
                else:
                    pkg_in_archive[pkg.name].append(pkg)
                # provides
                for provide in pkg.provides:
                    if provide not in pkg_in_archive:
                        pkg_in_archive[provide] = [pkg]
                    else:
                        pkg_in_archive[provide].append(pkg)

    # checkdep
    for packages in release.all_packages.values():
        for pkg in packages:
            for dep_group in pkg.dependencies:
                if not dep_group:
                    continue

                if ignore_noexist and True not in [pkg_in_archive.has_key(p[0].split(':')[0]) for p in dep_group]:
                    continue

                if ge_only and '>=' not in [dep[1] for dep in dep_group]:
                    continue

                try:
                    for depon, rel, version in dep_group:
                        depon_name = depon.split(':')[0]
                        deppkgs = pkg_in_archive.get(depon_name, [])
                        if not rel and deppkgs:
                            raise DependMatch()
                        else:
                            for deppkg in deppkgs:
                                if rel == '>=':
                                    ok = deppkg.binary_version >= version
                                elif rel == '<=':
                                    ok = deppkg.binary_version <= version
                                elif rel == '>=':
                                    ok = deppkg.binary_version >= version
                                elif rel in ['==', '=']:
                                    ok = deppkg.binary_version == version
                                elif rel in ['>', '>>']:
                                    ok = deppkg.binary_version > version
                                elif rel in ['<', '<<']:
                                    ok = deppkg.binary_version < version
                                else:
                                    raise Exception(
                                        'unknown relation: %s' % rel)
                                if ok:
                                    raise DependMatch()
                except DependMatch:
                    continue
                logger.error('%s:%s(%s) 有未满足的依赖:%s' %
                             (pkg.name, pkg.arch, pkg.version, dep_group))
    return True


def main(argv=None):
    """
    check dependencies of packages in an archive
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')
    checkdep(os.path.abspath(args['<suite>']),
             [os.path.abspath(suite) for suite in args['--extra']],
             args['--ignore-noexist'],
             args['--ge-only']
             )
    return 0



# encoding: utf-8
'''
Created on 2016-6-23

@author: xiewei

复制已有源中的某个suite，同时生成reprepro的db

'''

from ..contrib import docopt
import os
import logging

logger = logging.getLogger('archive_man')

cmd_doc = """
Usage:
   archive_man copy <source> <destdir> -s <suite> [-a <arch>...] [-c <components>...]

Options:
   -h, --help   show this help.
   -s, --suite=<suite>  codename of suite.
   -a, --arch=<arch>    choosed architectures, May be specified multiple
                        times. [default: i386 amd64 armhf arm64]
   -c, --component=<components>
                        components of local archive, May be specified
                        multiple times. [default: main restricted universe multiverse]
"""

command = 'copy'

def _verify_args(args):
    data = {}
    data['arch'] = ' '.join(args['--arch'])
    data['suite'] = args['--suite']
    data['component'] = ' '.join(args['--component'])
    if args['<source>'].find(':/') > 0:
        data['source'] = args['<source>']
    else:
        data['source'] = 'file://' + os.path.abspath(args['<source>'])
    data['destdir'] = os.path.expanduser(args['<destdir>'])
    return data


def _init_config(data):
    basedir = data['destdir']
    confdir = os.path.join(basedir, 'conf')
    try:
        os.makedirs(confdir, mode=0o775)
    except:
        pass
    if not os.path.isdir(confdir):
        raise Exception('cannot create config directory: "%s"' % confdir)
    with open(os.path.join(confdir, 'distributions'), 'w') as f:
        f.write("""Codename: %(suite)s
Architectures: %(arch)s
Components: %(component)s
Update: - %(suite)s
""" % data)
    with open(os.path.join(confdir, 'updates'), 'w') as f:
        f.write("""Name: %(suite)s
Method: %(source)s
Suite: %(suite)s
Components: %(component)s
Architectures: %(arch)s
""" % data)
    with open(os.path.join(confdir, 'options'), 'w') as f:
        f.write("""verbose
basedir .
""")
    logger.info(u'已生成reprepro配置文件')
    return


def copy_archive(data):
    _init_config(data)
    result = os.system('reprepro -b "%(destdir)s" update' % data)
    if result != 0:
        logger.error(u'复制软件源失败')
    else:
        logger.info(u'复制完成')
    return result


def main(argv=None):
    """
    copy archive (only one suite) to a local directory
    """
    args = docopt.docopt(cmd_doc, argv, help=True)
    data = _verify_args(args)
    return copy_archive(data)



# coding:utf-8

from __future__ import print_function

'''
Created on 2018-12-05

@author: xiewei
'''

cmd_doc = """
检查dists索引里的软件包和pool中的deb文件列表是否一致，输出多余或缺失的deb包路径
Usage: archive_man check <dir> [-s <suite>] [-m]

dir: 软件源目录，里面应该有dists和软件包目录（通常取名为pool）

options:
   -s, --suite=<suite>         仅仅检查指定系列的索引中缺失的文件
   -m, --md5                   检查仓库文件的md5是否与索引文件中一致
                               不一致的以前缀 ! 输出
   -h, --help                  show this help

"""

import os
from ..contrib import docopt
from . import utils

import logging

logger = logging.getLogger('archive_man')


def check(topdir, suite=None, check_md5=False):
    # TODO: check single suite
    index_dir = os.path.join(topdir, 'dists')
    if not os.path.isdir(index_dir):
        logger.error('%s 不是一个软件源目录', topdir)
        return False

    # find all Packages and Sources
    P_files = []
    S_files = []
    pool_files = []
    for folder, _, subfiles in os.walk(topdir):
        for subfile in subfiles:
            if folder.startswith(index_dir):
                if subfile == 'Packages':
                    filelist = P_files
                elif subfile == 'Sources':
                    filelist = S_files
                else:
                    continue
            else:
                filelist = pool_files
            filelist.append(os.path.join(folder, subfile))

    hash_table = {}
    keep_list = []
    for filepath in P_files:
        for package in utils.Packages.parse(filepath):
            package_abs_path = os.path.join(topdir, package.filename)
            hash_table[package_abs_path] = package.md5sum
            keep_list.append(package_abs_path)
            if os.path.islink(package_abs_path):
                # 链接目标保留
                keep_list.append(os.path.realpath(package_abs_path))
    for filepath in S_files:
        for source in utils.Sources.parse(filepath):
            for md5sum, _size, filepath in source.fileinfos:
                source_abs_path = os.path.join(topdir, filepath)
                hash_table[source_abs_path] = md5sum
                keep_list.append(source_abs_path)
                if os.path.islink(source_abs_path):
                    # 链接目标保留
                    keep_list.append(os.path.realpath(source_abs_path))

    # 对比
    for filepath in set(pool_files) - set(keep_list):
        print('+', filepath)

    for filepath in set(keep_list) - set(pool_files):
        print('-', filepath)

    if check_md5:
        for filepath, md5sum in hash_table.items():
            if not os.path.exists(filepath):
                continue
            if utils.file_hash(filepath) != md5sum:
                print('!', filepath)

    logger.info('检查完成')
    return True


def main(argv=None):
    """
    check missing or unnecessary debian packages in archive
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')

    check(topdir=os.path.abspath(args['<dir>']),
          suite=args['--suite'],
          check_md5=args['--md5']
          )
    return 0



# coding:utf8

'''
Created on 2017-2-22

@author: xiewei
'''

cmd_doc = """
Usage: archive_man merge [-d <topdir>] [-p <policy>] <source1> <source2>... -t <target> [-f] [-b]

source1: 第一个合并来源，应该是topdir中已存在的目录名
source2: 第二个合并来源，应该是topdir中已存在的目录名
...: 可提供更多合并来源

options:
   -d, --dir=<topdir>       dists目录的路径，默认为当前路径 [default: .]
   -p, --policy=<policy>    合并策略，默认取版本号最高 [default: version]
   -b, --binary             二进制包以包名而非source来判断是否同名包，这可以保留由不同版本source编译出的不同名称的包。
   -t, --target=<target>    合并后的系列名，注意如果是已存在的系列，里面的内容将会被替换（需要带 -f 选项）。
   -f, --force              如果目标系列已存在则会覆盖
   -h, --help               show this help

合并策略：
   对于相同源码名称的包，有三种合并策略：
   first：优先从参数中顺序靠前的系列中选取
   last：优先从参数中顺序靠后的系列中选取
   version：优先选源码版本号最高的

"""

import os
from ..contrib import docopt
from . import utils
import re

import logging

logger = logging.getLogger('archive_man')


def merge(topdir, froms, target, policy='version', binary=False, force=False):
    """
    合并多个系列中的Packages与Sources索引

    参数：
    topdir - 合并的系列应该属于同一个软件源，这里提供系列目录共同所在目录即 <xxx>/dists
    froms - 有序列表，内部元素是作为合并来源的系列名称
    target - 字符串，目标系列名称
    policy - 对于相同源码名称的包，有三种合并策略：
      first：从froms中顺序靠前的系列中选取
      last：从froms中顺序靠后的系列中选取
      version：选版本号高的
    """
    source_releases = [utils.Release.parse(os.path.join(
        topdir, series, 'Release')) for series in froms]
    # check target
    if os.path.exists(os.path.join(topdir, target)) and not force:
        logger.error('合并目标 "%s" 已存在' % target)
        return False

    def pkg_key(pkg, packages_arch):
        if binary:
            return pkg.name + ',' + pkg.arch + ',' + packages_arch
        else:
            return pkg.source

    logger.info('开始选择发布的软件包')
    best_versions = {}
    # 首先得到所选择的包的版本号
    for release in source_releases:
        this_versions = {}
        # 同一个系列中先确定最高版本
        for packages in release.all_packages.values() + release.all_sources.values():
            for pkg in packages:
                key = pkg_key(pkg, packages.arch)
                old_version = this_versions.get(key, '')
                if pkg <= old_version:
                    continue
                this_versions[key] = pkg.version
        # 然后按照指定策略更新至 best_version
        for key in this_versions:
            version = this_versions[key]
            old_version = best_versions.get(key, '')
            if policy == 'version':
                if utils.Version(version) <= old_version:
                    continue
            elif policy == 'first':
                if old_version:
                    continue
            best_versions[key] = version

    # 准备新的dist中的Packages
    new_packages = {}
    for release in source_releases:
        for fn in release.all_packages.keys() + release.all_sources.keys():
            newpath = os.path.join(topdir, target, fn)
            if not os.path.exists(newpath):
                try:
                    os.makedirs(os.path.dirname(newpath))
                except:
                    pass
            if fn not in new_packages:
                new_packages[fn] = utils.Packages(newpath)

    # 把选中的包填入对应的Packages中
    logger.info('生成合并后的Packages与Sources文件')
    for release in source_releases:
        for fn, packages in release.all_packages.items() + release.all_sources.items():
            for pkg in packages:
                if pkg == best_versions[pkg_key(pkg, packages.arch)]:
                    new_packages[fn][pkg.name] = pkg

    # 保存Packages文件
    for packages in new_packages.itervalues():
        packages.write()

    # 生成release文件
    logger.info('生成合并后的Release文件')
    new_release = utils.Release(os.path.join(topdir, target, 'Release'))
    for release in source_releases:
        new_release.merge_data(release.data)
    new_release.data['Suite'] = target
    new_release.write()
    logger.info('合并完成')
    return True


def main(argv=None):
    """
    merge 2 or more suites into a new suite in the same archive
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')

    merge(topdir=os.path.abspath(args['--dir']),
          froms=[args['<source1>']] + args['<source2>'],
          target=args['--target'],
          policy=args['--policy'],
          binary=args['--binary'],
          force=args['--force']
          )
    return 0



# coding:utf-8

'''
Created on 2019-01-22

@author: xiewei
'''

cmd_doc = """
修改软件源索引中的软件包路径，注意：默认不会移动对应路径的文件；源码文件重命名的操作有点复杂，暂不支持。
Usage: archive_man rename <dir> <origin> <new> [-f]
       archive_man rename <dir> --list=<file> [-f]

dir:     软件源目录，里面应该有dists和软件包目录（通常取名为pool）
origin:  需要修改的路径
new:     修改后的路径

options:
   -f, --file                  also rename the real file
   -l, --list=<file>           read list of files to rename from a file
                               the file content like:
                                  origin1,new1
                                  origin2,new2
   -h, --help                  show this help

"""

import os
import glob
import re
from ..contrib import docopt
from . import utils

import logging

logger = logging.getLogger('archive_man')


def rename(topdir, pairs, file=False):
    index_dir = os.path.join(topdir, 'dists')
    if not os.path.isdir(index_dir):
        logger.error('%s 不是一个软件源目录', topdir)
        return 1

    # parse package index
    for release_file in glob.glob(os.path.join(index_dir, '*', 'Release')):
        release = utils.Release.parse(release_file)
        release_changed = False
        for packages in release.all_packages.values():
            changed = False
            for package in packages:
                fn = package.filename
                prefix_expr = r'(Filename: )'

                if fn in pairs:
                    changed = True
                    logger.debug('Found %s in %s', fn, packages.filepath)
                    new_fn = pairs[fn]
                    # change package info
                    package.text, match_n = re.subn(r'^' + prefix_expr + re.escape(fn) + '$',
                                                    r'\g<1>' + new_fn,
                                                    package.text,
                                                    flags=re.M)
                    if match_n < 0:
                        logger.error('replace action not match any result')
                        return 1
                    if file:
                        logger.debug('rename file')
                        old = os.path.join(topdir, fn)
                        new = os.path.join(topdir, new_fn)

                        if os.path.exists(new):
                            logger.debug(
                                'new path already exists, maybe renamed before')
                            continue

                        new_dir = os.path.dirname(new)
                        if not os.path.exists(new_dir):
                            os.makedirs(new_dir)

                        if os.path.islink(old):
                            # 从链接目标复制
                            os.system('cp "%s" "%s"' % (old, new))
                        else:
                            # 重命名
                            os.rename(old, new)

            if changed:
                logger.debug('Rewriting: %s', packages.filepath)
                packages.write()
                release_changed = True

        # update Release
        if release_changed:
            logger.debug('Rewriting Release file: %s', release_file)
            release.write()
    return 0


def main(argv=None):
    """
    change filename of package in the archive indexes
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')

    name_pairs = {}

    if args['--list']:
        logger.info('processing name list...')

        with open(args['--list']) as f:
            for line in f.readlines():
                try:
                    stripped = line.strip()
                    if not stripped:
                        continue
                    a, b = stripped.split(',')
                    name_pairs[a.strip()] = b.strip()
                except:
                    logger.warning(
                        'Unable to parse filename pair from: %s', line)
                    pass

    else:
        name_pairs = {args['<origin>']: args['<new>']}

    return rename(os.path.abspath(args['<dir>']),
                  name_pairs,
                  file=args['--file']
                  )



# coding:utf-8

'''
Created on 2017-2-22

@author: xiewei
'''
import os
import sys
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

import re
pkg_fields = ['Package', 'Version', 'Architecture', 'Source',
              'Filename', 'Depends', 'Pre-depends', 'Provides', 'MD5sum', 'Size']
pkg_field_pattern = re.compile(r'^(?P<key>' + '|'.join(pkg_fields) + '): (?P<value>.+)',
                               re.M)
src_field_pattern = re.compile(r'^(?P<key>Package|Version|Directory): (?P<value>.+)',
                               re.M)
source_version_pattern = re.compile(r'(.+) \((.+)\)')
files_pattern = re.compile(r'^ (\w{32})\s+(\d+) (.+)', re.M)
dependency_pattern = re.compile(r'\s*(\S+) \((\S+) (\S+)\)')

# cmp mixin
PY3 = sys.version_info[0] >= 3
if PY3:
    def cmp(a, b):
        return (a > b) - (a < b)
    # mixin class for Python3 supporting __cmp__

    class PY3__cmp__:
        def __eq__(self, other):
            return self.__cmp__(other) == 0

        def __ne__(self, other):
            return self.__cmp__(other) != 0

        def __gt__(self, other):
            return self.__cmp__(other) > 0

        def __lt__(self, other):
            return self.__cmp__(other) < 0

        def __ge__(self, other):
            return self.__cmp__(other) >= 0

        def __le__(self, other):
            return self.__cmp__(other) <= 0
else:
    class PY3__cmp__:
        pass


def read_url(url):
    if url.startswith('file://'):
        url = url[7:]
    if '://' not in url:
        # as file
        return open(url).read()
    else:
        import urllib
        f = urllib.urlopen(url)
        return f.read()


class Release(object):
    """
    Release文件的内容，提供读取、保存等功能
    """

    def __init__(self, filepath):
        self.filepath = filepath
        self.data = OrderedDict()
        self.files = []
        self.packages_files = {}
        self.sources_files = {}

    def _parse(self):
        self.content = read_url(self.filepath)
        for k, v in re.findall(r'(\w+): ?(.+$)', self.content, re.M):
            self.data[k] = v

        self.files = [fileinfo[2]
                      for fileinfo in re.findall(files_pattern, self.content)]

        return

    @staticmethod
    def parse(release_file):
        obj = Release(release_file)
        obj._parse()
        return obj

    @property
    def all_packages(self):
        self.load_index('Packages')
        return self.packages_files

    @property
    def all_sources(self):
        self.load_index('Sources')
        return self.sources_files

    def load_index(self, name='Packages'):
        if name == 'Packages':
            index_list = self.packages_files
            index_class = Packages
        elif name == 'Sources':
            index_list = self.sources_files
            index_class = Sources
        else:
            raise NotImplementedError()

        if index_list:
            return
        for fn in self.files:
            if os.path.basename(fn) == name:
                fpath = os.path.join(os.path.dirname(self.filepath), fn)
            elif os.path.basename(fn) == name + '.gz':
                # gzipped
                fpath = os.path.join(os.path.dirname(self.filepath), fn)
                fn = fn.rsplit('.', 1)[0]
            else:
                continue

            if fn in index_list:
                # 已经统计的
                continue
            if not os.path.isfile(fpath):
                # 已经删除了的索引文件就不要管了
                continue
            if os.path.exists(fpath):
                index_list[fn] = index_class.parse(fpath)
        return

    def write(self):
        conf = 'APT::FTPArchive::Release {'
        for k, v in self.data.items():
            if k == 'Date':
                continue
            conf += '%s "%s";' % (k, v)
        conf += '};'

        # write conf
        import tempfile
        tmpconf = tempfile.mktemp('.conf')
        with open(tmpconf, 'w') as f:
            f.write(conf)
        # generate Release
        topdir = os.path.dirname(self.filepath)
        os.system(
            'rm -f "%(top)s"/InRelease "%(top)s"/Release.gpg "%(top)s"/Release' % {'top': topdir})

        content = os.popen('apt-ftparchive -c %(conf)s release %(top)s' % {'conf': tmpconf,
                                                                           'top': topdir
                                                                           }
                           ).read()
        with open(os.path.join(topdir, 'Release'), 'w') as f:
            f.write(content)

        from .sign import sign_file
        sign_file(topdir)
        return

    def merge_data(self, other_data):
        """
        merge release data to current
        """
        # special: Components, Architectures
        for key in other_data:
            other_value = other_data.get(key, '')
            if key in ['Components', 'Architectures']:
                new_set = set(self.data.get(key, '').split()
                              ) | set(other_value.split())
                self.data[key] = ' '.join(new_set)
            else:
                self.data[key] = other_value


class Packages(object):
    """
    Packages文件的内容，读取、解析包列表、保存
    """

    def __init__(self, filepath):
        self.filepath = filepath
        match = re.search(r'binary-(.*)/Packages', self.filepath)
        if match:
            self.arch = match.groups()[0]
        elif re.search('source/Sources', self.filepath):
            self.arch = 'src'
        else:
            self.arch = ''
        self.data = {}

    def _parse(self):
        if self.filepath.endswith('.gz'):
            import gzip
            f = gzip.open(self.filepath)
            self.filepath = self.filepath.rsplit('.', 1)[0]
        else:
            f = open(self.filepath)
        sections = f.read().strip().split('\n\n')
        for section in sections:
            if not section.strip():
                continue
            package = Package(section)
            # 同一个Packages里还有重复的，所以需要保留版本号最高的那个
            old_version = self.data.get(package.name, '')
            if package > old_version:
                self.data[package.name] = package
        f.close()
        return self.data

    @staticmethod
    def parse(packages_file):
        obj = Packages(packages_file)
        obj._parse()
        return obj

    @staticmethod
    def zip_packages(packagesfile, content=None):
        """
        根据Packages生成Packages.gz,Packages.bz2
        """
        if not content:
            with open(packagesfile, 'rb') as f:
                content = f.read()
        # gz and bz2
        import gzip
        zfile = gzip.open(packagesfile + '.gz', mode='wb')
        zfile.write(content)
        zfile.close()
        import bz2
        bzfile = bz2.BZ2File(packagesfile + '.bz2', 'wb')
        bzfile.write(content)
        bzfile.close()
        return

    def write(self, newpath=None, backup=''):
        """
        包列表写入Packages，并生成Packages.gz,Packages.bz2
        """
        filepath = newpath or self.filepath
        # create a origin backup
        if backup and os.path.exists(filepath):
            os.rename(filepath, filepath + '.' + backup)
        # write new
        with open(filepath, 'w') as f:
            for pkg_name in sorted(self.data.keys()):
                f.write(str(self.data[pkg_name]) + '\n\n')
        self.zip_packages(filepath)
        return

    def __setitem__(self, key, item):
        self.data[key] = item

    def __getitem__(self, key):
        return self.data[key]

    def __iter__(self):
        for v in self.data.values():
            yield v


class Package(PY3__cmp__, object):
    """
    Packages 文件中的单个记录
    """
    __slots__ = ['text', 'data']

    def __init__(self, text):
        self.text = text
        self.data = dict(re.findall(pkg_field_pattern, self.text))
        if 'Source' in self.data:
            source = self.data['Source']
            try:
                # 有的包版本号与源码版本号不一样
                sourcename, sourceversion = re.match(
                    source_version_pattern, source).groups()
                self.data['Source'] = sourcename
                self.data['SourceVersion'] = sourceversion
            except:
                self.data['SourceVersion'] = self.data['Version']
        else:
            self.data['Source'] = self.data['Package']
            self.data['SourceVersion'] = self.data['Version']

    @property
    def name(self):
        return self.data['Package']

    @property
    def source(self):
        return self.data['Source']

    @property
    def version(self):
        return self.data['SourceVersion']

    @property
    def binary_version(self):
        return Version(self.data['Version'])

    @property
    def arch(self):
        return self.data['Architecture']

    @property
    def filename(self):
        return self.data['Filename']

    @property
    def md5sum(self):
        return self.data['MD5sum']

    def __str__(self):
        return self.text

    def __cmp__(self, other):
        return Version(self.version).__cmp__(other)

    @property
    def provides(self):
        l = []
        for i in self.data.get('Provides', '').split(','):
            j = i.strip()
            if j:
                l.append(j)
        return l

    @property
    def dependencies(self):
        dep_list = []
        for key in ['Depends', 'Pre-Depends']:
            for dep1 in self.data.get(key, '').split(','):
                dep_group = []
                for dep in dep1.split('|'):
                    try:
                        dep_group.append(
                            re.match(dependency_pattern, dep).groups())
                    except AttributeError:
                        depon = dep.strip()
                        if depon:
                            dep_group.append((depon, '', ''))
                dep_list.append(dep_group)
        return dep_list


class Sources(Packages):
    def _parse(self):
        if self.filepath.endswith('.gz'):
            import gzip
            f = gzip.open(self.filepath)
            self.filepath = self.filepath.rsplit('.', 1)[0]
        else:
            f = open(self.filepath)
        sections = f.read().strip().split('\n\n')
        for section in sections:
            if not section.strip():
                continue
            source = Source(section)
            self.data[source.name] = source
        f.close()
        return self.data

    @staticmethod
    def parse(sources_file):
        obj = Sources(sources_file)
        obj._parse()
        return obj


class Source(Package):
    __slots__ = []

    def __init__(self, text):
        self.text = text

        self.data = dict(re.findall(src_field_pattern, self.text))
        self.data['Source'] = self.data['Package']
        self.data['SourceVersion'] = self.data['Version']
        self.data['files'] = re.findall(files_pattern, self.text)

    @property
    def files(self):
        directory = self.data['Directory']
        return [directory + '/' + fileinfo[2] for fileinfo in self.data['files']]

    @property
    def fileinfos(self):
        directory = self.data['Directory']
        return [(md5, size, directory + '/' + filename) for md5, size, filename in self.data['files']]

    @property
    def arch(self):
        return 'src'


class Version(PY3__cmp__, object):
    def __init__(self, value):
        self.version = value

    def __repr__(self, *args, **kwargs):
        return self.version

    def __cmp__(self, other):
        import apt
        try:
            other_version = other.version
        except:
            other_version = str(other)
        return apt.apt_pkg.version_compare(self.version, other_version)


def strip_packages(packagesfile):
    """
    remove lower version from Packages file
    """
    packages = Packages.parse(packagesfile)
    packages.write()
    return


def file_hash(filepath, hash_name='md5'):
    import hashlib
    h = hashlib.new(hash_name)
    f = open(filepath, 'r')
    while True:
        # 每次读取1M放到 data 中
        data = f.read(1024 * 1024)
        size = len(data)
        if not size:
            # 数据流已经读取完毕
            break
        # 校验器校验一下
        h.update(data)
    f.close()
    return h.hexdigest()



# encoding: utf-8

from __future__ import print_function

'''
Created on 2016-6-14

@author: xiewei
'''

import os
import sys
from ..contrib import gnupg
from ..contrib import docopt
from . import config
from . import logging

logger = logging.getLogger('archive_man')

cmd_doc = """
Usage:
   archive_man key import <keyfile> [-p <password>]
   archive_man key delete <keyid>
   archive_man key list

Options:
   -h, --help                   show this help
   -p, --password=<password>    save password in the config file,
                                then automatically use it while signing file
"""


def add_key(keyfile):
    gpg = gnupg.GPG(gnupghome=config.GPGHOME)
    with open(keyfile) as f:
        result = gpg.import_keys(f.read())
    try:
        assert result.stderr.find('IMPORT_OK') >= 0, result.stderr
    except AssertionError:
        logger.error(u'导入出错: %s' % result.stderr)
        sys.exit(1)


def delete_key(keyid):
    gpg = gnupg.GPG(gnupghome=config.GPGHOME)
    result = gpg.delete_keys(keyid, secret=True)
    try:
        assert str(result) == 'ok', result.stderr
    except AssertionError:
        logger.error(u'删除出错: %s' % result.stderr)
        sys.exit(1)


def list_key():
    gpg = gnupg.GPG(gnupghome=config.GPGHOME)
    result = gpg.list_keys(secret=True)
    for secret in result:
        print(secret['fingerprint'], ', '.join(secret['uids']))


def main(argv=None):
    """
    import PGP key for signing
    """
    args = docopt.docopt(cmd_doc, argv, help=True)
    if args['import']:
        add_key(args['<keyfile>'])
        if args['--password']:
            config.conf.set('app', 'gpgpass', args['--password'])
            config.store()
    elif args['delete']:
        delete_key(args['<keyid>'])
    elif args['list']:
        list_key()
    else:
        pass
    return 0



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

with open('VERSION', 'r') as fh:
    version = fh.read().strip()

setuptools.setup(
    name='apt-archive-tools',
    version=version,
    author='Xie Wei',
    author_email='xw.master@live.cn',
    description='Apt archive toolkit',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/sir-xw/apt-archive-tools',
    packages=setuptools.find_packages(),
    license='GPLv2',
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: POSIX :: Linux',
    ],
    package_data={
        'apt_archive_tools.lib': ['default.conf']
    },
    entry_points={
        'console_scripts': [
            'archive-man = apt_archive_tools.archive_man:main'
        ],
        'apt_archive_tools.commands': [
            'key = apt_archive_tools.lib.key:main',
            'sign = apt_archive_tools.lib.sign:main',
            'copy = apt_archive_tools.lib.copy_archive:main',
            'local-copy = apt_archive_tools.lib.local_copy:main',
            'publish = apt_archive_tools.lib.publish:main',
            'merge = apt_archive_tools.lib.merge:main',
            'strip = apt_archive_tools.lib.strip:main',
            'diff = apt_archive_tools.lib.diff:main',
            'check = apt_archive_tools.lib.check:main',
            'checkdep = apt_archive_tools.lib.checkdep:main',
            'rename = apt_archive_tools.lib.rename:main'
        ]
    }
)



# encoding: utf-8
'''
Created on 2016-6-23

@author: xiewei

把本地软件源再复制一份，pool目录里的包将会是源文件的硬链接，以节省空间

'''

from ..contrib import docopt
import os
import logging

logger = logging.getLogger('archive_man')

cmd_doc = """
Usage:
   archive_man local-copy <source-dir> <dest-dir>

Options:
   -h, --help   show this help
"""


def copy_archive(source, dest):
    if not os.path.isdir(os.path.join(source, 'pool')):
        logger.error(u'源目录不是一个软件源')
        return 1
    os.makedirs(dest, mode=0o755)
    # copy files
    for sub in os.listdir(source):
        result = os.system('cp -ar %s "%s" "%s"' % ('-l' if sub == 'pool' else '',
                                                    os.path.join(source, sub),
                                                    os.path.join(dest, sub)
                                                    ))
        if result == 0:
            logger.info(u'复制了 %s' % sub)
        else:
            logger.error(u'复制 %s 时出错' % sub)
            return result

    return 0


def main(argv=None):
    """
    just like cp, but files in pool/ will be a hardlink to source
    """
    args = docopt.docopt(cmd_doc, argv, help=True)
    return copy_archive(args['<source-dir>'], args['<dest-dir>'])



# coding:utf-8

from __future__ import print_function

'''
Created on 2017-2-22

@author: xiewei
'''

cmd_doc = """
Usage: archive_man diff [-t <type>] [-f|-c] <source1> <source2>
       archive_man diff -m <archive1> <archive2>

source1: 第一个对比目录，里面应该存在Release文件
source2: 第二个对比目录，里面应该存在Release文件
archive1: 第一个对比仓库，里面应该存在dists子目录
archive2: 第二个对比仓库，里面应该存在dists子目录

options:
   -t, --type=<type>        对比类型：source或binary [default: source]
   -f, --file               输出文件名而不是版本号
   -c, --compare            比较版本号大小
   -m, --md5                比较两个仓库中同名文件的md5值
   -h, --help               show this help

"""

import os
import glob
from ..contrib import docopt
from . import utils

import logging

logger = logging.getLogger('archive_man')


def diff(dist1, dist2, method='source', listfile=False, compare=False):
    release1 = utils.Release.parse(os.path.join(dist1, 'Release'))
    release2 = utils.Release.parse(os.path.join(dist2, 'Release'))

    logger.info('开始分析软件包列表')

    versions1 = {}
    versions2 = {}

    for vs, r in (versions1, release1), (versions2, release2):
        # 同一个系列中先确定最高版本
        if method == 'source':
            all_packages = r.all_sources.values()
        else:
            all_packages = r.all_packages.values()

        for packages in all_packages:
            for pkg in packages:
                pkgname = pkg.name + ', ' + pkg.arch
                old_version = vs.get(pkgname, '')
                if pkg <= old_version:
                    continue
                vs[pkgname] = pkg

    logger.info('开始比较')
    # 输出对比结果
    for pkgname in set(versions1.keys()) - set(versions2.keys()):
        if listfile:
            print(pkgname, ',', filepath(versions1[pkgname]))
        else:
            print(pkgname, ',', versions1[pkgname].version)
    for pkgname in set(versions1.keys()) & set(versions2.keys()):
        cmp_res = versions1[pkgname].__cmp__(versions2[pkgname])
        if compare:
            if cmp_res == 0:
                cmp_char = '='
            elif cmp_res < 0:
                cmp_char = '<'
            else:
                cmp_char = '>'
            print(pkgname, ',', versions1[pkgname].version, ',',
                  cmp_char, ',', versions2[pkgname].version)
        elif cmp_res == 0:
            continue
        elif listfile:
            print(pkgname, ',', filepath(versions1[pkgname]), ',', filepath(versions2[pkgname]))
        else:
            print(pkgname, ', ,', versions1[pkgname].version, ',', versions2[pkgname].version)
    for pkgname in set(versions2.keys()) - set(versions1.keys()):
        if listfile:
            print(pkgname, ', ,', filepath(versions2[pkgname]))
        else:
            print(pkgname, ', , ,', versions2[pkgname].version)

    logger.info('比较完成')
    return 0


def diff_md5(archive1, archive2):
    hash_table1 = {}
    hash_table2 = {}

    for topdir, hash_table in ((archive1, hash_table1), (archive2, hash_table2)):
        index_dir = os.path.join(topdir, 'dists')
        if not os.path.isdir(index_dir):
            logger.error('%s 不是一个软件源目录', topdir)
            return 1
        for release_file in glob.glob(os.path.join(index_dir, '*', 'Release')):
            release = utils.Release.parse(release_file)
            for packages in release.all_packages.values() + release.all_sources.values():
                for package in packages:
                    if isinstance(package, utils.Source):
                        for md5, _size, filepath in package.fileinfos:
                            hash_table[filepath] = md5
                    else:
                        hash_table[package.filename] = package.md5sum
    # compare
    for fn in set(hash_table1.keys()) & set(hash_table2.keys()):
        md5_1 = hash_table1[fn]
        md5_2 = hash_table2[fn]
        if md5_1 != md5_2:
            print(fn, ',', md5_1, ',', md5_2)

    return 0


def filepath(package):
    try:
        return package.filename
    except:
        return '|'.join(package.files)


def main(argv=None):
    """
    compare the package list of two dists
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')

    if args['--md5']:
        return diff_md5(args['<archive1>'], args['<archive2>'])
    else:
        return diff(dist1=args['<source1>'],
                    dist2=args['<source2>'],
                    method=args['--type'],
                    listfile=args['--file'],
                    compare=args['--compare'],
                    )



# coding:utf-8

'''
Created on 2017-03-24

@author: xiewei
'''

cmd_doc = """
从软件源中删除已经不在dists索引里的包，减少其占用空间
Usage: archive_man strip <dir> [-b <backupdir>] [-d] [-i] [-p <pattern>...] [-f <pattern-file>]

dir: 软件源目录，里面应该有dists和软件包目录（通常取名为pool）

options:
   -b, --backup=<backupdir>    多余的包不会被删除，而是移动到指定的备份目录中
   -d, --dry                   提示哪些文件会被删除，但并不执行
   -h, --help                  show this help
   -i, --index                 双向删除：同时会将已经不存在于pool中的包从索引中删除
   -p, --pattern=<pattern>     额外将路径匹配pattern的软件包删除
   -f, --from-file=<pattern-file>
                               从文件中读取pattern

"""

import os
import glob
import re
from ..contrib import docopt
from . import utils

import logging

logger = logging.getLogger('archive_man')


def strip(topdir, backup, index, dryrun=False, pattern=None):
    """
    从软件源中删除已经不在dists索引里的包
    """
    index_dir = os.path.join(topdir, 'dists')
    if not os.path.isdir(index_dir):
        logger.error('%s 不是一个软件源目录', topdir)
        return 1

    if backup and not os.path.exists(backup):
        os.makedirs(backup)

    # find all Packages and Sources
    pool_files = set()
    keep_list = set()

    logger.debug('Collecting files')
    for folder, _, subfiles in os.walk(topdir):
        for subfile in subfiles:
            if folder.startswith(index_dir):
                continue
            else:
                pool_files.add(os.path.join(folder, subfile))

    # parse package index
    for release_file in glob.glob(os.path.join(index_dir, '*', 'Release')):
        release = utils.Release.parse(release_file)
        release_changed = False
        for packages in release.all_packages.values() + release.all_sources.values():
            new = utils.Packages(packages.filepath)
            changed = False
            for package in packages:
                if isinstance(package, utils.Source):
                    file_list = package.files
                else:
                    file_list = [package.filename]
                for filename in file_list:
                    package_abs_path = os.path.join(topdir, filename)
                    if pattern and pattern.search(filename):
                        logger.debug('Matched file: %s', filename)
                        if index:
                            logger.debug('Remove %s from %s',
                                            package.name, packages.filepath)
                            changed = True
                        break
                    if index:
                        if package_abs_path in pool_files:
                            new[package.name] = package
                        else:
                            logger.debug('Missing file: %s', package_abs_path)
                            logger.debug('Remove %s from %s',
                                         package.name, packages.filepath)
                            changed = True
                            break
                    keep_list.add(package_abs_path)
                    if os.path.islink(package_abs_path):
                        # 链接目标保留
                        keep_list.add(os.path.realpath(package_abs_path))

            if index and changed:
                if dryrun:
                    logger.debug(
                        'Index file need to rewrite: %s', new.filepath)
                else:
                    logger.debug('Rewriting: %s', new.filepath)
                    new.write()
                    release_changed = True

        # update Release
        if release_changed:
            logger.debug('Rewriting Release file: %s', release_file)
            release.write()

    # 开始删除或移动
    for filepath in pool_files - keep_list:
        if dryrun:
            logger.debug('Find an unnecessary file: %s', filepath)
        elif backup:
            old_dir, _ = os.path.split(filepath)
            back_dir = backup + old_dir.replace(topdir, '', 1)
            if not os.path.exists(back_dir):
                os.makedirs(back_dir)
            cmd = 'mv "%s" "%s"' % (filepath, back_dir)
            logger.debug('Excecuting: %s', cmd)
            assert os.system(cmd) == 0
        else:
            logger.debug('Removing: %s', filepath)
            try:
                os.remove(filepath)
            except:
                logger.error('remove failed: %s', filepath)

    if dryrun:
        logger.info('测试模式完成，没有改动任何文件')
    else:
        logger.info('裁剪完成')
    return 0


def main(argv=None):
    """
    remove unnecessary files from pool/ if local archive
    """
    args = docopt.docopt(cmd_doc, argv, help=True, version='1.0')

    backup = args['--backup']
    if not backup:
        backupdir = None
    else:
        backupdir = os.path.abspath(backup)

    patterns = set(args['--pattern'])
    pattern_file = args['--from-file']
    if pattern_file:
        if not os.path.isfile(pattern_file):
            logger.error('pattern 文件 "%s" 不存在', pattern_file)
            return 1
        with open(pattern_file) as f:
            for pattern in f.read().split('\n'):
                if not pattern.strip():
                    continue
                if pattern.startswith('#'):
                    continue
                patterns.add(pattern)

    return strip(topdir=os.path.abspath(args['<dir>']),
                 backup=backupdir,
                 index=args['--index'],
                 dryrun=args['--dry'],
                 pattern=re.compile(
                     '|'.join('(%s)' % p for p in patterns)) if patterns else None
                 )



'''
Created on 2016-6-14

@author: xiewei
'''

import os
try:
    from ConfigParser import SafeConfigParser
except ImportError:
    from configparser import SafeConfigParser

conf = SafeConfigParser()
conf.read([os.path.join(os.path.dirname(__file__), 'default.conf'),
           '/etc/apt-tools.conf',
           os.path.expanduser('~/.config/apt-tools.conf')
           ])

GPGHOME = os.path.expanduser(conf.get('app', 'gpghome'))
GPGKEYPASS = conf.get('app', 'gpgpass')

options = {'suite': conf.get('options', 'suite'),
           'arch': conf.get('options', 'arch')
           }


def store(path=os.path.expanduser('~/.config/apt-tools.conf')):
    path_dir = os.path.dirname(path)
    if not os.path.exists(path_dir):
        os.makedirs(path_dir)
    with open(path,'wb') as f:
        conf.write(f)
    return True
